var http, https, url;

http = require("http");
https = require("https");
url = require("url");

exports.create = function (options) {
    options = options || {};
    // TODO: Check if types match.
    options.requestHeaders = options.requestHeaders || [];
    options.responseHeaders = options.responseHeaders || [];
    return function (req, res) {
        var parsedUrl, proxyUrl, proxyReq, proxyResHandler;
        parsedUrl = url.parse(req.url, true);
        if (parsedUrl.query.hasOwnProperty("url")) {
            proxyUrl = url.parse(parsedUrl.query.url);
            var proxyOptions = {
                "host": proxyUrl.host,
                "path": proxyUrl.pathname,
                "method": req.method,
                "headers": {}
            };
            // Copy white-listed HTTP request headers.
            Object.keys(req.headers).forEach(function (requestHeader) {
                options.requestHeaders.forEach(function (optionsRequestHeader) {
                    var matcher;
                    matcher = requestHeader.match(new RegExp(optionsRequestHeader, "i"));
                    if (matcher !== null) {
                        proxyOptions.headers[matcher[0]] = req.headers[matcher[0].toLowerCase()];
                    }
                });
            });
            // url.parse does not parse localhost correctly.
            if (proxyOptions.host.indexOf("localhost") !== -1) {
                proxyOptions.host = proxyOptions.host.split(":")[0];
            }
            // url.parse does not parse complex pathnames correctly.
            proxyOptions.path += req.url.split(proxyOptions.path)[1];
            // Different protocol, different module, but same response handler.
            proxyResHandler = function (proxyRes) {
                var resHeaders;
                // Copy white-listed HTTP response headers.
                resHeaders = {};
                Object.keys(proxyRes.headers).forEach(function (responseHeader) {
                    options.responseHeaders.forEach(function (optionsResponseHeader) {
                        var matcher;
                        matcher = responseHeader.match(new RegExp(optionsResponseHeader, "i"));
                        if (matcher !== null) {
                            resHeaders[matcher[0]] = proxyRes.headers[matcher[0].toLowerCase()];
                        }
                    });
                });
                // Respect status code.
                res.writeHead(proxyRes.statusCode, resHeaders);
                proxyRes.on("data", function (chunk) {
                    res.write(chunk);
                });
                proxyRes.on("end", function () {
                    res.end();
                });
            };
            if (proxyUrl.protocol === "https:") {
                proxyOptions.port = proxyUrl.port || 443;
                proxyReq = https.request(proxyOptions, proxyResHandler);
            } else {
                proxyOptions.port = proxyUrl.port || 80;
                proxyReq = http.request(proxyOptions, proxyResHandler);
            }
            proxyReq.on("error", function (e) {
                res.writeHead(500);
                res.end("Captain, we crashed: " + e.code + "\n");
            });
            // I tried to use pipes here, but sometimes they had a hiccup.
            req.on("data", function (chunk) {
                proxyReq.write(chunk);
            });
            req.on("end", function () {
                proxyReq.end();
            });
        } else {
            res.writeHead(400);
            res.end("\"url\", did you pass it?\n");
        }
    };
};
